#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <QFileDialog>
#include <QTextStream>
#include <QDebug>
#include <QGraphicsItem>
#include <QFontMetrics>
#include <QMap>
#include <QColor>
#include <qmath.h>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    scene = new QGraphicsScene(this);
    ui->graphicsView->setScene(scene);
    parseFile("/home/mm/brcm6348pins.txt");
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::on_pushButton_clicked()
{
    this->parseFile(QFileDialog::getOpenFileName(this, "", QDir::homePath(), "PINOUT file (*.txt)"));

}


void MainWindow::parseFile(QString fn)
{
    QRect rect(0,0,80,80);
    try {
        QFile file(fn);
        if (file.open(QFile::ReadOnly)) {
            QTextStream inStream(&file);
            QString line;
            int i= 0;
            QMap <QString, QColor> colors;
            do {
                 line = inStream.readLine();
                 if (line.isNull())
                     break;
                 QString name, pad;
                 QStringList list = line.split('\t');

                 if (list.size() == 2) {
                    name = list[0];
                    pad = list[1];
                    QString rowStr, colStr;
                    int col = 0, row = 0;
                    for (int j = 0; j<pad.size(); j++) {
                        if (pad[j].isLetter())  {
                            rowStr.append(pad[j]);
                        } else if (pad[j].isNumber()) {
                            colStr.append(pad[j]);
                        }
                    }
                    col = colStr.toInt() -1;

                    for (int j = (rowStr.size()-1) ; j >= 0; j--) {
                        row += ((int)rowStr.at(j).toAscii() - 64) + j*24 -1;
                    }

                    QColor color;
                    if (!name.split('_').isEmpty()) {
                        QString prefix = name.split('_').first();
                        if (colors.contains(prefix)) {
                            color = colors[prefix];
                        } else {
                            color.setRed(qrand()%255);
                            color.setGreen(qrand()%255);
                            color.setBlue(qrand()%255);
                            colors[prefix] = color;
                        }
                    }

                    QGraphicsRectItem *rectItem =
                    scene->addRect(0+col*rect.width(),
                                   0+row*rect.height(),
                                   rect.width(),
                                   rect.height());
                    rectItem->setPen(QPen(Qt::black));
                    rectItem->setBrush(QBrush(color));

                    QColor textColor = color;
                    textColor.setRed(255-textColor.red());
                    textColor.setGreen(255-textColor.green());
                    textColor.setBlue(255-textColor.blue());
                    pad.prepend("<center>");
                    pad.append("</center>");

                    int weight = color.red() + color.blue() + color.green();
                    if (weight < 400)
                        textColor = QColor(Qt::white);
                    else
                        textColor = QColor(Qt::black);

                    QGraphicsTextItem *textItem = scene->addText("");
                    textItem->setHtml(pad);
                    textItem->setDefaultTextColor(textColor);
                    textItem->setTextWidth(rect.width());
                    textItem->setPos(0+col*rect.width(),
                                     0+row*rect.height());


                    name.replace('_', '\n');
                    name.prepend("<center>");
                    name.append("</center>");
                    QGraphicsTextItem *text2Item = scene->addText("");
                    text2Item->setHtml(name);
                    text2Item->setDefaultTextColor(textColor);
                    text2Item->setTextWidth(rect.width());
                    text2Item->setPos(0+col*rect.width(),
                                     0+row*rect.height()+20);


                } else {
                    qWarning() << QString("unparsable line:%1").arg(i);
                }
                i++;
            } while (!line.isNull());
            file.close();

            // draw cols
            for (int i = 0; i<26; i++) {
                if ((i > 5) && (i < 21)) {
                    scene->addLine(i*rect.width(), 0, i*rect.width(), 5*rect.width());
                    scene->addLine(i*rect.width(), 25*rect.width(), i*rect.width(), 30*rect.width());
                } else {
                    scene->addLine(i*rect.width(), 0, i*rect.width(), 30*rect.width());
                }
            }

            // draw rows
            for (int i = 0; i<30; i++) {
                if ((i > 5) && (i < 25)) {
                    scene->addLine(0, i*rect.height(), 5*rect.width(), i*rect.height());
                    scene->addLine(21*rect.width(), i*rect.height(), 25*rect.width(), i*rect.height());
                } else {
                    scene->addLine(0, i*rect.height(), 26*rect.width(), i*rect.height());
                }
            }

            QString text = "<center><font size=40>This picture was generated by Marton Miklos in 2010</font></center>";
            QGraphicsTextItem *nameIt = scene->addText("");
            nameIt->setHtml(text);
            nameIt->setPos(scene->width() /2, scene->height());

            scene->update();
            QImage *img = new QImage(QSize(scene->width(), scene->height()), QImage::Format_ARGB32);
            QPainter *imagePainter =  new QPainter(img);
            scene->render(imagePainter);
            img->save(QDir::homePath()+"/pinout.png");
            delete imagePainter;
            delete img;
        } else {
            throw QString("OPen failed");
        }
    } catch (QString e) {
        ui->label->setText(e);
    }
}


